package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.module.entity.MaterialImageInfo;
import cn.com.ctop.common.module.entity.SysUser;
import cn.com.ctop.common.module.service.IMaterialImageInfoService;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.common.module.vo.*;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouReportDailyImageMapper;
import cn.com.ctop.toutiao.modules.report.mapper.BytedanceReportMaterialDailyMapper;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.jeecg.modules.ctop.service.IImageReportDailyService;
import org.jeecg.modules.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ImageReportDailyServiceImpl implements IImageReportDailyService {
    @Autowired
    private IMaterialImageInfoService imageInfoService;
    @Autowired
    private BytedanceReportMaterialDailyMapper bytedanceReportMaterialDailyMapper;
    @Autowired
    private KuaishouReportDailyImageMapper kuaishouReportDailyImageMapper;
    @Autowired
    private ISysUserService sysUserService;


    @Override
    public Map<String,Object> bytedanceImageCost(JSONObject data) {
        Map<String,Object>result = new HashMap<>();
        String startDate = data.getString("startDate");
        if(null == startDate||startDate.trim().equals("")){
            startDate = null;
        }
        String endDate = data.getString("endDate");
        if(null == endDate||endDate.trim().equals("")){
            endDate = null;
        }
        JSONArray accountList = data.getJSONArray("accountList");
        if(null==accountList||accountList.isEmpty()){
            accountList = null;
        }
        String code = data.getString("code");
        if(null == code||code.trim().equals("")){
            code = null;
        }
        Integer pageSize = data.getInteger("pageSize");
        Integer pageNumber = data.getInteger("pageNumber");
        if(null == pageNumber||pageNumber ==0){
            pageNumber = 1;
        }
        if(null == pageSize||pageSize ==0){
            pageSize = 10;
        }
        PageHelper.startPage(pageNumber,pageSize);
        List<BytedanceImageCostVO> vos = bytedanceReportMaterialDailyMapper.bytedanceImageCost(startDate,endDate,accountList,code);
        PageInfo<BytedanceImageCostVO> pageInfo = new PageInfo<>(vos);
        if(null!=vos&&!vos.isEmpty()){
            List<BytedanceImageCostVO>setVos = new ArrayList<>();
            for(BytedanceImageCostVO vo:vos){
                MaterialImageInfo imageInfo = imageInfoService.getByCode(vo.getSignature());
                if(null!=imageInfo){
                    vo.setUserId(imageInfo.getUserId());
                    SysUser user = sysUserService.getById(imageInfo.getUserId());
                    vo.setUserName(user.getUsername());
                }else{
                    vo.setUserName("-");
                }
                setVos.add(vo);
            }
            pageInfo.setList(setVos);
        }
        result.put("data",pageInfo);
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }

    @Override
    public SheetInfoVo getbytedanceImageCostVo(JSONObject data) {
        SheetInfoVo sheetInfoVo = new SheetInfoVo<BytedanceImageCostVOEntity>();
        String startDate = data.getString("startDate");
        if(null == startDate||startDate.trim().equals("")){
            startDate = null;
        }
        String endDate = data.getString("endDate");
        if(null == endDate||endDate.trim().equals("")){
            endDate = null;
        }
        JSONArray accountList = data.getJSONArray("accountList");
        if(null==accountList||accountList.isEmpty()){
            accountList = null;
        }
        String code = data.getString("code");
        if(null == code||code.trim().equals("")){
            code = null;
        }
        List<BytedanceImageCostVO> vos = bytedanceReportMaterialDailyMapper.bytedanceImageCost(startDate,endDate,accountList,code);
        List<BytedanceImageCostVOEntity>details = new ArrayList<>();
        if(null!=vos&&!vos.isEmpty()){
            for(BytedanceImageCostVO vo:vos){
                BytedanceImageCostVOEntity entity = new BytedanceImageCostVOEntity(vo);
                MaterialImageInfo imageInfo = imageInfoService.getByCode(vo.getSignature());
                if(null!=imageInfo){
                    entity.setUserId(imageInfo.getUserId());
                    SysUser user = sysUserService.getById(imageInfo.getUserId());
                    entity.setUserName(user.getUsername());
                }else{
                    entity.setUserName("-");
                }
                details.add(entity);
            }
        }
        sheetInfoVo.setDetails(details);
        return sheetInfoVo;
    }

    @Override
    public Map<String, Object> kuaishouImageCost(JSONObject data) {
        Map<String,Object>result = new HashMap<>();
        String startDate = data.getString("startDate");
        if(null == startDate||startDate.trim().equals("")){
            startDate = null;
        }
        String endDate = data.getString("endDate");
        if(null == endDate||endDate.trim().equals("")){
            endDate = null;
        }
        JSONArray accountList = data.getJSONArray("accountList");
        if(null==accountList||accountList.isEmpty()){
            accountList = null;
        }
        String code = data.getString("code");
        if(null == code||code.trim().equals("")){
            code = null;
        }
        Integer pageSize = data.getInteger("pageSize");
        Integer pageNumber = data.getInteger("pageNumber");
        if(null == pageNumber||pageNumber ==0){
            pageNumber = 1;
        }
        if(null == pageSize||pageSize ==0){
            pageSize = 10;
        }
        PageHelper.startPage(pageNumber,pageSize);
        List<KuaishouImageCostVO> vos = kuaishouReportDailyImageMapper.imageCost(startDate,endDate,accountList,code);
        PageInfo<KuaishouImageCostVO> pageInfo = new PageInfo<>(vos);
        if(null!=vos&&!vos.isEmpty()){
            List<KuaishouImageCostVO>setVos = new ArrayList<>();
            for(KuaishouImageCostVO vo:vos){
                MaterialImageInfo imageInfo = imageInfoService.getByCode(vo.getSignature());
                if(null!=imageInfo){
                    vo.setUserId(imageInfo.getUserId());
                    SysUser user = sysUserService.getById(imageInfo.getUserId());
                    vo.setUserName(user.getUsername());
                }else{
                    vo.setUserName("-");
                }
                setVos.add(vo);
            }
            pageInfo.setList(setVos);
        }
        result.put("data",pageInfo);
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }

    @Override
    public SheetInfoVo getkuaishouImageCostVo(JSONObject data) {
        SheetInfoVo sheetInfoVo = new SheetInfoVo<KuaishouImageCostVOEntity>();
        String startDate = data.getString("startDate");
        if(null == startDate||startDate.trim().equals("")){
            startDate = null;
        }
        String endDate = data.getString("endDate");
        if(null == endDate||endDate.trim().equals("")){
            endDate = null;
        }
        JSONArray accountList = data.getJSONArray("accountList");
        if(null==accountList||accountList.isEmpty()){
            accountList = null;
        }
        String code = data.getString("code");
        if(null == code||code.trim().equals("")){
            code = null;
        }
        sheetInfoVo.setSheetName("快手图片消耗统计");
        List<KuaishouImageCostVO> vos = kuaishouReportDailyImageMapper.imageCost(startDate,endDate,accountList,code);
        List<KuaishouImageCostVOEntity> details = new ArrayList<>();
        if(null!=vos&&!vos.isEmpty()){
            for (KuaishouImageCostVO vo:vos) {
                KuaishouImageCostVOEntity entity = new KuaishouImageCostVOEntity(vo);
                MaterialImageInfo imageInfo = imageInfoService.getByCode(vo.getSignature());
                if(null!=imageInfo){
                    entity.setUserId(imageInfo.getUserId());
                    SysUser user = sysUserService.getById(imageInfo.getUserId());
                    entity.setUserName(user.getUsername());
                }else{
                    entity.setUserName("-");
                }
                details.add(entity);
            }
        }
        sheetInfoVo.setDetails(details);
        return sheetInfoVo;
    }
}
